#include <iostream>

#include <zypp/ZYpp.h>
#include <zypp/ZYppFactory.h>
#include <zypp/PathInfo.h>
#include <zypp/Capability.h>

#include <zypp/RepoManager.h>
#include <zypp/media/MediaManager.h>
#include <zypp/Resolvable.h>
#include <zypp/ResPool.h>
#include <zypp/Repository.h>
#include <zypp/RepoManager.h>
#include <zypp/RepoInfo.h>
#include <zypp/repo/RepoException.h>
#include <zypp/target/rpm/RpmException.h>
#include <zypp/parser/ParseException.h>
#include <zypp/Pathname.h>
#include <zypp/sat/Pool.h>
#include <zypp/target/rpm/RpmDb.h>
#include <zypp/target/rpm/RpmHeader.h>
#include <zypp/target/rpm/librpmDb.h>
#include <zypp/base/LogControl.h>
#include <zypp/base/String.h>

#include <zypp/base/Logger.h>
#include <zypp/ui/Selectable.h>
#include <zypp/Pattern.h>
using namespace::std;
using namespace::zypp;
zypp::ZYpp::Ptr zyppPtr = zypp::ZYppFactory::instance().getZYpp();

std::vector<zypp::sat::Solvable> *
zypp_get_packages_by_name (const char *package_name, const zypp::ResKind kind, zypp::ResPool pool)
{
    std::vector<zypp::sat::Solvable> *v = new std::vector<zypp::sat::Solvable> ();

    for (zypp::ResPool::byIdent_iterator it = pool.byIdentBegin (kind, package_name);
            it != pool.byIdentEnd (kind, package_name); it++) {
        v->push_back (it->satSolvable ());
    }

    return v;
}

void zypp_build_pool()
{
    zypp::Pathname sysRoot( "/" );
    zyppPtr->initializeTarget( sysRoot, false );
    zyppPtr->target()->load();

    zypp::sat::Pool pool = zypp::sat::Pool::instance ();
    zypp::RepoManager manager;
    std::list <zypp::RepoInfo> repos;
    repos = std::list<zypp::RepoInfo>(manager.repoBegin(),manager.repoEnd());

    bool force = false;
    int i = 0;
    for (std::list <zypp::RepoInfo>::iterator it = repos.begin(); it != repos.end(); it++, i++) {
        zypp::RepoInfo repo (*it);
        // Erase old solv file
        pool.reposErase(repo.alias ());
        cout << "refresh repo :" <<repo.alias () <<endl;
        // Refreshing metadata
        manager.refreshMetadata (repo, force ?
                zypp::RepoManager::RefreshForced :
                zypp::RepoManager::RefreshIfNeededIgnoreDelay);
        manager.buildCache (repo, force ?
                zypp::RepoManager::BuildForced :
                zypp::RepoManager::BuildIfNeeded);
        manager.loadFromCache (repo);
    }
}

int main(int argc, char **argv) 
{
    zypp_build_pool();
    if (argc != 2)
    {
        cout << "Usage:" << argv[0] << " pkgname" <<endl;
        return 1;
    }
    zypp::ResPool respool = zyppPtr->pool ();

    std::vector<zypp::sat::Solvable> *v;
    zypp::sat::Solvable newest;

    v = zypp_get_packages_by_name (argv[1], zypp::ResKind::pattern, respool);
    for (std::vector<zypp::sat::Solvable>::iterator it = v->begin (); it != v->end (); it++) {
        if (*it == zypp::sat::Solvable::noSolvable)
            continue;
        if (newest == zypp::sat::Solvable::noSolvable) {
            newest = *it;
        } else if (it->edition().match (newest.edition()) > 0) {
            newest = *it;
        }
    }
    delete (v);
    zypp::PoolItem item = zypp::ResPool::instance ().find (newest);
    item.status ().setToBeInstalled (zypp::ResStatus::USER);
    Pattern::constPtr pattern = asKind<Pattern>(item.resolvable());
    Pattern::Contents contents = pattern->contents();
    for_(sit, contents.selectableBegin(), contents.selectableEnd()) {
        const ui::Selectable & s = **sit;
        if (s.kind().asString() == "package")
            cout << s.name() <<endl;
    }
    return EXIT_SUCCESS;
}
